### Wrapper R script to perform single variant analysis
dyn.load(paste("../lib/epacts",.Platform$dynlib.ext, sep=""))

pheno <- as.double(read.table(phenof)[,2])
pheno <- pheno - min(pheno,na.rm=T)
n <- length(pheno)

if ( covf == "NULL" ) {
  cov <- NULL;
} else {
  cov <- as.matrix(read.table(covf)[,-1])
}
cov <- cbind(rep(1,n),cov)  
ind <- as.integer(read.table(indf)[,2])-1

source(paste(test,".R",sep="",collapse=NULL))
nr <- length(groups)

func <- match.fun(test)
for(i in 1:nr) {
  ## load the genotype matrix from groupwise info
  G <- .Call("readVcfGroup",vcf,field,ind,groups[[i]],sepchr)
  NS <- rowSums(!is.na(G))
  AC <- rowSums(G,na.rm=T)
  if ( flip ) {  ## flip major/minor, AC == MAC
    fids <- which(AC>NS)
    G[fids,] <- 2-G[fids,]
    AC[fids] <- NS[fids]*2 - AC[fids]
    MAC <- AC
  }
  else {
    MAC <- NS - abs(NS-AC) # if AC < NS, NS-NS+AC=AC, if AC>NS, NS-AC+NS=2NS-AC
  }
  MAF <- MAC / (NS + NS)
  vids <- which((MAF >= minAF) & ( MAF <= maxAF ) & ( MAC >= minAC ))
  genos <- G[vids,,drop=FALSE]  ## informative genotypes

  collapse <- as.double(colSums(genos,na.rm=T) > 0)
  sum.collapse <- sum(collapse,na.rm=T)
  nona.collapse <- sum(!is.na(collapse),na.rm=T)
  if ( nona.collapse > 0 ) {
    maf.collapse <- sum.collapse/nona.collapse
  } else {
    maf.collapse <- NA
  }

  r <- func()
  if ( i == 1 ) {
    out <- matrix(NA,nr,length(r$cname)+6)
    colnames(out) <- c("NS","FRAC_BURDEN","NUM_ALL_VARS","NUM_PASS_VARS","NUM_SING_VARS","PVALUE",r$cname)
  }
  #print(r)
  out[i,] <- c(nona.collapse,maf.collapse,nrow(G),nrow(genos),sum(AC==1),r$p,r$add)
}
rownames(out) <- paste(regions,gnames,sep="_")
print(warnings())

.Call("writeMatrix",out,outf)
