### Wrapper R script to perform single variant analysis

args <- commandArgs(trailingOnly=TRUE)
bindir <- args[2]
phenof <- args[3]
covf <- args[4]
indf <- args[5]
vcf <- args[6]
region <- args[7]
outf <- args[8]
field <- args[9]
minAF <- as.double(args[10])
minMAC <- as.integer(args[11])
minCallRate <- args[12]
minRSQ <- args[13]
passOnly <- (args[14] == "TRUE")
test <- args[15]

setwd(bindir)
#print(bindir)

dyn.load(paste("../lib/epacts",.Platform$dynlib.ext, sep=""))

#print(minAF)
  
pheno <- as.double(read.table(phenof)[,2])
pheno <- pheno - min(pheno,na.rm=T)
if ( covf == "NULL" ) {
  cov <- NULL;
} else {
  cov <- as.matrix(read.table(covf)[,-1])
}

ind <- as.integer(read.table(indf)[,2])-1

G <- .Call("readVcf",vcf,region,field,passOnly,ind,NULL)
m <- nrow(G)
if ( m > 0 ) {
  n <- ncol(G)
  cov <- cbind(rep(1,n),cov)  
  NS <- rowSums(!is.na(G))
  AC <- rowSums(G,na.rm=T)
  MAF <- 0.5 - abs(AC/NS/2 - 0.5)
  CR <- NS/n
  sqAC <- rowSums(G*G,na.rm=T)
  varAC <- sqAC/NS - AC*AC/NS/NS
  if ( minRSQ > 0 ) {
    rsq <- (sqAC-AC*AC/(NS+1e-30))/(AC - AC*AC/(2*NS+1e-30))
    rsq[rsq>1] <- 1
    vids <- which((varAC > 0) & (MAF >= minAF) & ( AC >= minMAC ) & ( NS + NS - AC >= minMAC) & (CR >= minCallRate) &&  (rsq >= minRSQ))
  }
  else {
    vids <- which((varAC > 0) & (MAF >= minAF) & ( AC >= minMAC ) & ( NS + NS - AC >= minMAC) & (CR >= minCallRate))
  }
  genos <- G[vids,,drop=FALSE]

  #print(sum(MAF >= minAF))
  #print(MAF)
  #print(minAF)
  #print(sum(NS+NS-AC>2))
  #print(sum(CR >= minCallRate))

  source(paste(test,".R",sep="",collapse=NULL))
  #print(test)

  func <- match.fun(test)
  r <- func()

  out <- matrix(NA,m,5+(minRSQ>0)+ncol(r$add))
  
  out[,1] <- NS
  out[,2] <- AC
  out[,3] <- CR
  out[,4] <- MAF
  if ( minRSQ > 0 ) {
    colnames(out) <- c("NS","AC","CALLRATE","MAF","RSQ","PVALUE",r$cname);    
    out[,5] <- rsq
    out[vids,6] <- r$p
    out[vids,7:ncol(out)] <- r$add
  }
  else {
    colnames(out) <- c("NS","AC","CALLRATE","MAF","PVALUE",r$cname);    
    out[vids,5] <- r$p
    out[vids,6:ncol(out)] <- r$add
  }
  rownames(out) <- rownames(G)
  print(warnings())
  .Call("writeMatrix",out,outf)
} else {
  write.table(NULL,outf,row.names=F,col.names=F)  
}

