## Core functions of pugwash to perform association

##################################################################
## SINGLE VARIANT TEST
## INPUT VARIABLES:
##   n        : total # of individuals
##   NS       : number of called samples
##   AC       : allele count
##   MAF      : minor allele frequency
##   vids     : indices from 1:n after AF/AC threshold
##   genos    : genotype matrix (after AF/AC threshold)
## EXPECTED OUTPUT : list(p, addcols, addnames) for each genos row
##   p        : p-value
##   addcols  : additional columns (with proper column names)
##################################################################

## single.b.lrt:
## Likelihood ratio test
## Use built-in glm logistic regression to perform association
## By:  Clement Ma
##
## KEY FEATURES :SIMPLE, BUT MAY BE SLOW
## TRAITS  : BINARY
## RETURNS : PVALUE, BETA, SEBETA, ZSTAT
## MISSING VALUES : IGNORED
single.b.lrt <- function() {
  #cname <- NULL
  m <- nrow(genos)
  p <- rep(NA,m)
  cname <- c("NULL_DEVIANCE","FULL_DEVIANCE");
  add <- matrix(NA,m,2)
  #add <- matrix(NA,m,3) ## BETA, SEBETA, TSTAT
  
  # Model without genotype
  r.nogeno <- glm(pheno~cov-1, family="binomial")
  if ( m > 0 ) {
    for(i in 1:m) {
      # add back genotype
      r <- update(r.nogeno, ~ . + genos[i,])
      if ( ( r$converged ) && ( ! r$boundary ) ) {
        #p[i] <- anova(r, r.nogeno, test="Chisq")[2,"Pr(>Chi)"]
        p[i] <- pchisq(r.nogeno$deviance-r$deviance, df=1, lower.tail=F)
        add[i,] <- c(r.nogeno$deviance,r$deviance)
      }
      else {
        p[i] <- NA
      }
    }
  }
  return(list(p=p,add=add,cname=cname))
}
