## Core functions of pugwash to perform association

##################################################################
## SINGLE VARIANT TEST
## INPUT VARIABLES:
##   n        : total # of individuals
##   NS       : number of called samples
##   AC       : allele count
##   MAF      : minor allele frequency
##   vids     : indices from 1:n after AF/AC threshold
##   genos    : genotype matrix (after AF/AC threshold)
## EXPECTED OUTPUT : list(p, addcols, addnames) for each genos row
##   p        : p-value
##   addcols  : additional columns (with proper column names)
##################################################################

## single.b.logit.wald :
## Use built-in wald logistic regression to perform association
## KEY FEATURES :SIMPLE, BUT MAY BE SLOW
## TRAITS  : BINARY
## RETURNS : PVALUE, BETA, SEBETA, ZSTAT
## MISSING VALUES : IGNORED
single.b.wald <- function() {
  cname <- c("BETA","SEBETA","ZSTAT")
  m <- nrow(genos)
  p <- rep(NA,m)
  add <- matrix(NA,m,3) ## BETA, SEBETA, TSTAT
  if ( m > 0 ) {
    for(i in 1:m) {
      r <- glm(pheno~genos[i,]+cov-1,family="binomial")
      if ( ( r$converged ) && ( ! r$boundary ) ) {
        s <- summary(r)$coefficients[1,]
        p[i] <- s[4]
        add[i,] <- s[1:3]
      }
      else {
        p[i] <- NA
        add[i,] <- rep(NA,3)
      }
    }
  }
  return(list(p=p,add=add,cname=cname))
}
