#!/bin/bash
mkdir --p out/
DIR=`dirname $0`
VCF=${DIR}/1000G_exome_chr20_example_softFiltered.calls.vcf.gz
PED=${DIR}/1000G_dummy_pheno.ped
OUT=out/test
GRP=${DIR}/1000G_exome_chr20_example_softFiltered.calls.sites.anno.grp 

mkdir --p out/
echo "Performing EPACTS Binary Single Variant Score Test.."
./epacts single --vcf ${VCF} --ped ${PED} --minAF 0.001 --chr 20 --pheno DISEASE --cov AGE --cov SEX --test b.score --out ${OUT}.single.b.score --run 5 --anno

echo "Creating Empirical Kinship matrix..."
./epacts single --vcf ${VCF} --ped ${PED} --minAF 0.01 --chr 20 --pheno QT --cov AGE --cov SEX --test q.emmax --out ${OUT}.single.q.emmax.kinf --run 5 --kinOnly

echo "Running EMMAX single variant test..."
./epacts single --vcf ${VCF} --ped ${PED} --minAF 0.001 --chr 20 --pheno QT --cov AGE --cov SEX --test q.emmax --out ${OUT}.single.q.emmax --kinf ${OUT}.single.q.emmax.kinf --run 5 --anno

echo "Taking the site list from the VCF file";
zcat ${VCF} | cut -f 1-8 | bgzip -c > ${DIR}/1000G_exome_chr20_example_softFiltered.calls.sites.vcf.gz

echo "Annotate the VCF file";
./epacts anno --in ${DIR}/1000G_exome_chr20_example_softFiltered.calls.sites.vcf.gz --out ${DIR}/1000G_exome_chr20_example_softFiltered.calls.sites.anno.vcf.gz

echo "Create the input group file for burden test";
./epacts makegroup --vcf ${DIR}/1000G_exome_chr20_example_softFiltered.calls.sites.anno.vcf.gz --out ${GRP} -format epacts -nonsyn

echo "Performing SKAT-O test";
./epacts group --vcf ${VCF} --ped ${PED} --maxAF 0.05 --groupf ${GRP} --pheno QT --cov AGE --cov SEX --test skat --out ${OUT}.gene.skat --skat-o --unit 100 --run 5