package epacts;
use base qw/Exporter/;

## Variables and methods shared across the package
@EXPORT_OK = qw(@chrs @szchrs @cumszchrsMb $tabixDir parsePheno getMosixCmd schr2nchr %ichrs);

$tabixDir = "tabix-0.2.5";

BEGIN {
## Variables below are based on GRCh37
    @chrs = (1..22,"X","Y","MT");
    @szchrs = qw(249250621 243199373 198022430 191154276 180915260 171115067 159138663 146364022 141213431 135534747 135006516 133851895 115169878 107349540 102531392 90354753 81195210 78077248 59128983 63025520 48129895 51304566 155270560 59373566 16569);
    %ichrs = ();
    
    @cumszchrsMb = (0);
    for(my $i=0; $i < @chrs; ++$i) {
	push(@cumszchrsMb,$szchrs[$i]/1e6+$cumszchrsMb[$i]);
	$ichrs{$chrs[$i]} = $i;
    }
}

## parse phenotype value, resolving missing values
sub parsePheno {
    my ($p,$missing) = @_;
    if ( $p eq $missing ) {
	return "NA";
    }
    elsif ( $p =~ /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/ ) {
	return $p;
    }
    else {
	return "NA";  ## non-numeric values are converted to NA
    }
}

## convert command to mosix command
sub getMosixCmd {
    my ($cmd,$nodes) = @_;
    return "mosrun -E/tmp -i -t -j$nodes sh -c '$cmd'";
}

## convert string chromosome to integer (compatible to PLINK)
sub schr2nchr {
    my $chr = shift;
    $chr =~ s/^chr// if ( substr($chr,0,3) eq "chr" );
    if ( $chr =~ /^\d+$/ ) { return $chr; }
    elsif ( $chr eq "X" ) { return 23; }
    elsif ( $chr eq "Y" ) { return 24; }
    elsif ( $chr eq "MT" ) { return 25; }
}
