##################################################################
## GENE-LEVEL BURDEN TEST
## INPUT VARIABLES: 
##   n        : total # of individuals
##   genos    : genotype matrix for each gene
##   NS       : number of called samples for each marker
##   AC       : allele count for each marker
##   MAC      : minor allele count for each marker
##   MAF      : minor allele frequency
##   vids     : indices from 1:n after AF/AC threshold
## EXPECTED OUTPUT : list(p, addcols, addnames) for each genos row
##   p        : p-value
##   add      : additional column values
##   cname    : additional column names
##################################################################

#library("CompQuadForm");
#source("../lib/CompQuadForm/R/davies.R")
#source("../lib/CompQuadForm/R/farebrother.R")
#source("../lib/CompQuadForm/R/imhof.R")
#source("../lib/CompQuadForm/R/liu.R")
#source("../lib/CompQuadForm/R/zzz.R")
#dyn.load(paste("../lib/CompQuadForm/src/CompQuadForm",.Platform$dynlib.ext, sep=""))
dyn.load(paste("../lib/SKAT",.Platform$dynlib.ext, sep=""))
source("../lib/SKAT/R/Function.R")
source("../lib/SKAT/R/KMTest_Linear.R")
source("../lib/SKAT/R/KMTest_Logistic_VarMatching.R")
source("../lib/SKAT/R/KMTest_Optimal_VarMatching.R")
source("../lib/SKAT/R/Main_SSD.R")
source("../lib/SKAT/R/Null_Model.R")
source("../lib/SKAT/R/Power.R")
source("../lib/SKAT/R/SSD.R")
source("../lib/SKAT/R/Kernel.R")
source("../lib/SKAT/R/KMTest_Logistic.R")
source("../lib/SKAT/R/KMTest_Optimal.R")
source("../lib/SKAT/R/Main.R")
source("../lib/SKAT/R/Null_Model_EP.R")
source("../lib/SKAT/R/Power_Corr.R")
source("../lib/SKAT/R/SmallSample.R")
source("../lib/SKAT/R/SKAT_CompQuadForm.R")

## gene.q.skat() : SKAT implementation
## KEY FEATURES : 0/1 collapsing variable ~ rare variants
## TRAITS  : QUANTITATIVE (GAUSSIAN)
## RETURNS : SKAT_PVALUE, SKAT_QSTAT, rSKAT_PVALUE, rSKAT_QSTAT, mdSKAT_PVALUE, mdSKAT_QSTAT, SKATO_PVALUE, SKATO_RHO, rSKATO_PVALUE, rSKATO_RHO for SKAT, rSKAT, SKAT with Madsen Browning type weights, SKAT_O, rSKAT_O
## MISSING VALUE : IMPUTED AS MAJOR ALLELES


## SKAT
## binary vs quantitiative
## weighted vs flat
## optimal vs original
gene.skat <- function() {
  #library(SKAT)
  #print(packageDescription("SKAT"))
  if ( skatOptimal ) {
    cname <- c("STATRHO");
  }
  else {
    cname <- c("QSTAT");
  }
  
  #cname <- c("QTSTAT","rSKAT_PVALUE","rSKAT_QSTAT","mdSKAT_PVALUE","mdSKAT_QSTAT","SKATO_PVALUE","SKATO_RHO","rSKATO_PVALUE","rSKATO_RHO")
  ## flip ref/alt to minor/major
  flip <- which(AC>NS)
  genos[flip,] <- 2-genos[flip,]
  m <- nrow(genos)
  if ( m > 0 ) {
    if ( binaryFlag ) {
      obj <- SKAT_Null_Model(pheno~cov-1,out_type="D",Adjustment=FALSE) # Dichotomous outcome
      #obj <- SKAT_Null_Model(pheno~cov-1,out_type="C");
    }
    else {
      obj <- SKAT_Null_Model(pheno~cov-1,out_type="C") # Continuous outcome
    }
    # Default SKAT with linear weighted kernel and weights = beta(MAF,1,25)
    if ( skatOptimal ) {
      if ( skatFlat ) {
        r <- SKAT(t(genos),obj,kernel="linear",method="optimal",impute.method="fixed")
      }
      else {
        r <- SKAT(t(genos),obj,kernel="linear.weighted",method="optimal",weights.beta=betas,impute.method="fixed")
      }

      p <- r$p.value
      if ( m == 1 ) {
        rho <- NA
      }
      else {
        rho <- r$param$rho_est
        if ( length(rho) > 1 ) {
          rho <- "NA"  ## temporary solution
        }
      }
      return(list(p=p,add=c(rho),cname=cname))
    }
    else {
      if ( skatFlat ) {
        r <- SKAT(t(genos),obj,kernel="linear",method="davies",impute.method="fixed")
      }
      else {
        r <- SKAT(t(genos),obj,kernel="linear.weighted",method="davies",weights.beta=betas,impute.method="fixed")
        #stop("foo")                
      }
      Q <- r$Q
      p <- ifelse(r$p.value==0,r$param$liu_pval,r$p.value)
      rm(r)
      gc()
      return(list(p=p,add=c(Q),cname=cname))
    }
  }
  else {
    return(list(p=NA,add=rep(NA,length(cname)),cname=cname))
  }
}
